-- [F/A] derelky


baleIdManager = {}
baleIdManager.ModName = g_currentModName
baleIdManager.ModDirectory = g_currentModDirectory
baleIdManager.eventName = {}
baleIdManager.Version = "1.0.0.0"


addModEventListener(baleIdManager)

function baleIdManager.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Baler, specializations)
end

function baleIdManager:loadMap(name)
	print("--- loading baleIdManager V".. baleIdManager.Version .. " | ModName " ..tostring(baleIdManager.ModName).. " ---")
end

function baleIdManager.registerFunctions(vehicleType)
    SpecializationUtil.registerFunction(vehicleType, "getBaleFarm", baleIdManager.getBaleFarm)
end

function baleIdManager.registerOverwrittenFunctions(vehicleType)
    SpecializationUtil.registerOverwrittenFunction(vehicleType, "createBale",     baleIdManager.createBale)
    SpecializationUtil.registerOverwrittenFunction(vehicleType, "dropBale",     baleIdManager.dropBale)
end

function baleIdManager.registerEventListeners(vehicleType)
	local functionNames = {	"onRegisterActionEvents", "onDraw", "onLoad", "saveToXMLFile", "onWriteStream","onReadStream" }
	
	for _, functionName in ipairs(functionNames) do
		SpecializationUtil.registerEventListener(vehicleType, functionName, baleIdManager)
	end
end

		
function baleIdManager:onLoad(savegame)
	local spec = self.spec_baleidmanager	

	spec.FarmID = self:getOwnerFarmId()
	spec.FarmIDs = {}
	spec.ContractActive = false
	spec.balecounter = {}
	spec.isMultiplayer = g_currentMission.missionDynamicInfo.isMultiplayer
    for i=1,8,1 do	
		spec.balecounter[i] = 0
    end
	if savegame ~= nil then
		local xmlFile = savegame.xmlFile
		local FarmID = savegame.key..".baleIdManager#FarmID"
		local UseContr = savegame.key..".baleIdManager#UseContr"
		
		local i = 0
            while true do
                local key = string.format("%s.baleIdManager.Farm(%d)", savegame.key, i)
                if not hasXMLProperty(xmlFile, key) then
                    break
                end
                local farmbalecounter = getXMLInt(xmlFile, key.."#index")
                local balecount = Utils.getNoNil(getXMLInt(xmlFile, key.."#balecounter"),0)				
				spec.balecounter[farmbalecounter] = balecount
                i = i + 1
            end
		if spec.isMultiplayer then		
			spec.FarmID = Utils.getNoNil(getXMLInt(xmlFile, FarmID), self:getOwnerFarmId())
			spec.ContractActive = Utils.getNoNil(getXMLBool(xmlFile, UseContr), false)
		else
			spec.FarmID = FarmManager.SINGLEPLAYER_FARM_ID
			spec.ContractActive = false
		end
		baleIdManager:updateFarmID(self, spec.FarmID,spec.ContractActive, true)
	end
end

function baleIdManager:saveToXMLFile(xmlFile, key)
	if self.spec_baleidmanager ~= nil then
		setXMLInt(xmlFile, key.."#FarmID", self.spec_baleidmanager.FarmID)
		setXMLBool(xmlFile, key.."#UseContr", self.spec_baleidmanager.ContractActive)
		
		for i=0, 7,1 do
			local baleIdManagerKey = string.format("%s.Farm(%d)", key, i)
			setXMLInt(xmlFile, baleIdManagerKey.."#index", i+1)
			setXMLInt(xmlFile, baleIdManagerKey.."#balecounter", self.spec_baleidmanager.balecounter[i+1])
		end
	end
end

function baleIdManager:onRegisterActionEvents(isSelected, isOnActiveVehicle)
	
	local spec = self.spec_baleidmanager	
	if spec.isMultiplayer then
		spec.actionEvents = {}
		if isSelected then
			local _, actionEventId = self:addActionEvent(spec.actionEvents, 'BALEID_CHANGE', self, baleIdManager.action_baleId_Change, false, true, false, true, nil)
			g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_HIGH)
		end		
	end
end

function baleIdManager:requestFarmID(FarmID,contractstate)
	if FarmID ~= nil and contractstate ~=nil then
		baleIdManager:updateFarmID(self,FarmID,contractstate, false)
	end	
end

function baleIdManager:requestBalecounter(Balecounter)		
		self.spec_baleidmanager.balecounter=Balecounter
end

function baleIdManager:action_baleId_Change(actionName, keyStatus, arg3, arg4, arg5)
	self.spec_baleidmanager.FarmIDs ={}
	for i=1,8,1 do
		if g_currentMission.accessHandler:canFarmAccessOtherId(self:getActiveFarm(),i) then
			table.insert(self.spec_baleidmanager.FarmIDs,i)
		end
	end
	
    local dialog = g_gui:showDialog("baleIdManagerGui")
    if dialog ~= nil then
        dialog.target:FarmCallback(baleIdManager.requestFarmID,baleIdManager.requestBalecounter, self)
        dialog.target:loadFarmID(self.spec_baleidmanager.FarmID,self.spec_baleidmanager.FarmIDs,self.spec_baleidmanager.ContractActive)
        dialog.target:loadbalecounter(self.spec_baleidmanager.balecounter)
        dialog.target:changetext(baleIdManager.ModName)
    end
end

function baleIdManager:onDraw(isActiveForInput, isActiveForInputIgnoreSelection, isSelected)	
	local spec = self.spec_baleidmanager	
	local Farmname = spec.FarmID
	if spec.FarmID == self:getActiveFarm() then
		Farmname=g_i18n.modEnvironments[baleIdManager.ModName].texts.OwnFarm
	else
		Farmname=g_farmManager:getFarmById(spec.FarmID).name
	end
	
	if spec.isMultiplayer then
		g_currentMission:addExtraPrintText(g_i18n.modEnvironments[baleIdManager.ModName].texts.BaleIdCurrent .. string.format(" : %s",  Farmname));
		g_currentMission:addExtraPrintText(g_i18n.modEnvironments[baleIdManager.ModName].texts.BaleIdCounter1 .. string.format(" : %d",  spec.balecounter[spec.FarmID]));	
	end
end

function baleIdManager:createBale(superFunc, baleFillType, fillLevel)
    local spec = self.spec_baler
    if spec.knotingAnimation ~= nil then
        self:playAnimation(spec.knotingAnimation, spec.knotingAnimationSpeed, nil, true)
    end
    if spec.dummyBale.currentBale ~= nil then
        delete(spec.dummyBale.currentBale)
        spec.dummyBale.currentBale = nil
    end
    local t = spec.baleTypes[spec.currentBaleTypeId]
    local baleType = g_baleTypeManager:getBale(baleFillType, t.isRoundBale, t.width, t.height, t.length, t.diameter)
    local bale = {}
    bale.filename = baleType.filename--Utils.getFilename(baleType.filename, self.baseDirectory)	
    bale.time = 0
    bale.fillType = baleFillType
    bale.fillLevel = fillLevel
    if spec.baleUnloadAnimationName ~= nil then
        local baleRoot = g_i3DManager:loadSharedI3DFile(baleType.filename, nil, false, false)
        local baleId = getChildAt(baleRoot, 0)
        link(spec.baleAnimRoot, baleId)
        delete(baleRoot)
        bale.id = baleId
    end
    if self.isServer and spec.baleUnloadAnimationName == nil then
        local x,y,z = getWorldTranslation(spec.baleAnimRoot)
        local rx,ry,rz = getWorldRotation(spec.baleAnimRoot)
        local baleObject = Bale:new(self.isServer, self.isClient)
        baleObject:load(bale.filename, x,y,z,rx,ry,rz, bale.fillLevel)
        baleObject:setOwnerFarmId(self:getBaleFarm(), true)
        baleObject:register()
        baleObject:setCanBeSold(false)
        local baleJointNode = createTransformGroup("BaleJointTG")
        link(spec.baleAnimRoot, baleJointNode)
        setTranslation(baleJointNode, 0,0,0)
        setRotation(baleJointNode, 0,0,0)
        local constr = JointConstructor:new()
        constr:setActors(spec.baleAnimRootComponent, baleObject.nodeId)
        constr:setJointTransforms(baleJointNode, baleObject.nodeId)
        for i=1, 3 do
            constr:setRotationLimit(i-1, 0, 0)
            constr:setTranslationLimit(i-1, true, 0, 0)
        end
        constr:setEnableCollision(false)
        local baleJointIndex = constr:finalize()
        g_currentMission:removeItemToSave(baleObject)
        bale.baleJointNode = baleJointNode
        bale.baleJointIndex = baleJointIndex
        bale.baleObject = baleObject
    end
    table.insert(spec.bales, bale)
end

function baleIdManager:dropBale(superFunc, baleIndex)
    local spec = self.spec_baler
    local bale = spec.bales[baleIndex]
    if self.isServer then
        local baleObject
        if bale.baleJointIndex ~= nil then
            baleObject = bale.baleObject
            removeJoint(bale.baleJointIndex)
            delete(bale.baleJointNode)
            g_currentMission:addItemToSave(bale.baleObject)
        else
            baleObject = Bale:new(self.isServer, self.isClient)
            local x,y,z = getWorldTranslation(bale.id)
            local rx,ry,rz = getWorldRotation(bale.id)
            baleObject:load(bale.filename, x,y,z,rx,ry,rz, bale.fillLevel)
            baleObject:setOwnerFarmId(self:getBaleFarm(), true)
            baleObject:register()
        end
        if spec.lastBaleFillLevel ~= nil and #spec.bales == 1 then
            baleObject:setFillLevel(spec.lastBaleFillLevel)
            spec.lastBaleFillLevel = nil
        end
        baleObject:setCanBeSold(true)
        if baleObject.nodeId ~= nil then
            -- release bale
            local x,y,z = getWorldTranslation(baleObject.nodeId)
            local vx,vy,vz = getVelocityAtWorldPos(spec.baleAnimRootComponent, x,y,z)
            setLinearVelocity(baleObject.nodeId, vx,vy,vz)
        end
        g_farmManager:updateFarmStats(self:getLastTouchedFarmlandFarmId(), "baleCount", 1)
    end
    if bale.id ~= nil and entityExists(bale.id) then
        delete(bale.id)
    end
    g_i3DManager:releaseSharedI3DFile(bale.filename, nil, true)
	
	self.spec_baleidmanager.balecounter[self:getBaleFarm()] = self.spec_baleidmanager.balecounter[self:getBaleFarm()] + 1 
    table.remove(spec.bales, baleIndex)
end

function baleIdManager:onWriteStream(streamId, connection)
    if not connection:getIsServer() then 
		local spec = self.spec_baleidmanager;
		for i=1,8,1 do
			streamWriteInt32(streamId, spec.balecounter[i]);
		end
		streamWriteInt8(streamId, spec.FarmID)
		streamWriteBool(streamId, spec.ContractActive)
	end;
end;

function baleIdManager:onReadStream(streamId, connection)
    if connection:getIsServer() then
		local spec = self.spec_baleidmanager;
		for i=1,8,1 do		
			spec.balecounter[i] = streamReadInt32(streamId);
		end
		spec.FarmID  = streamReadInt8(streamId)
		spec.ContractActive  = streamReadBool(streamId)
		
	end;
end;

function baleIdManager:updateFarmID(obj,FarmIDnew,ContractActive, noEventSend)
	local spec =obj.spec_baleidmanager
	local FarmID = obj:getActiveFarm()
	if spec ~= nil  then
		if ContractActive then
			FarmID = FarmIDnew					
		end
		spec.FarmID =FarmID
		spec.ContractActive=ContractActive
	end
	baleIdManagerEvent.sendEvent(obj, FarmID,ContractActive, noEventSend)
end


function baleIdManager:getBaleFarm()
	local spec = self.spec_baleidmanager	
	if spec.isMultiplayer then
		local farmId = spec.FarmID
		if farmId ~= 0 then
			return farmId
		else
			return self:getActiveFarm()
		end
	else
		return self:getActiveFarm()
	end
end
-- MP Events

baleIdManagerEvent = {}
baleIdManagerEvent_mt = Class(baleIdManagerEvent, Event)

InitEventClass(baleIdManagerEvent, "baleIdManagerEvent")

function baleIdManagerEvent:emptyNew()
    local self = Event:new(baleIdManagerEvent_mt)
	self.className = "baleIdManagerEvent"
    return self
end
function baleIdManagerEvent:new(object, FarmID, ContractActive)
    local self = baleIdManagerEvent:emptyNew()
    self.object = object
	self.FarmID = FarmID
	self.ContractActive = ContractActive
    return self
end

function baleIdManagerEvent:readStream(streamId, connection)
	self.object = NetworkUtil.readNodeObject(streamId)
	self.FarmID  = streamReadInt8(streamId)
	self.ContractActive  = streamReadBool(streamId)
	self:run(connection)
end

function baleIdManagerEvent:writeStream(streamId, connection)
	NetworkUtil.writeNodeObject(streamId, self.object)
	streamWriteInt8(streamId, self.FarmID)
	streamWriteBool(streamId, self.ContractActive)
end

function baleIdManagerEvent:run(connection)
	baleIdManager:updateFarmID(self.object, self.FarmID, self.ContractActive, true)
    if not connection:getIsServer() then
        g_server:broadcastEvent(baleIdManagerEvent:new(self.object, self.FarmID,self.ContractActive), nil, connection, self.object)
    end
end

function baleIdManagerEvent.sendEvent(vehicle, FarmID,ContractActive, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(baleIdManagerEvent:new(vehicle, FarmID, ContractActive), nil, nil, vehicle)
		else
			g_client:getServerConnection():sendEvent(baleIdManagerEvent:new(vehicle, FarmID, ContractActive))
		end
	end
end
